# freq_str : freq_id mapping
FREQ_MAPPING = {
    "D": 1,  # daily
    "B": 2,  # business daily
    "W-FRI": 3,  # weekly (friday)
    "W": 3,  # weekly (friday)
    "M": 5,  # monthly
    "Q": 6,  # quarterly
    "2Q": 7,  # semi-annual
    "A": 8,  # annual
    "Y": 8,  # annual
}

FREQUENCY_TR_MAPPING = {
    "AYLIK": "M",
}
